<?php
   /*
      Descrição: Logic criada para centralizar as rotinas de hierarquia de serviço
      Autor: Laércio Augusto
      Data: 20/10/2014
   */
namespace Services\Logic;

use \Services\Logic\OrcamentoLogic;

Class HierarquiaServicoLogic{
    const TIPO_INFERIOR = 'INFERIOR';
    const TIPO_PRINCIPAL = 'PRINCIAPL';
    const TIPO_SEM_HIERARQUIA = 'SEM_HIERARQUIA';
    const STR_VAZIO = 'VAZIO';
    const STR_SIM = 'SIM';
    const STR_NAO = 'NAO';

    private $nrseqorcamento;
    private $nrhierarquiaserv;
    private $nrorg;
    private $servicoPrincipal;
    private $cdprecobase;
    private $tipomedida;
    private $modeloorca;
    private $itemespelho;
    private $grupofatura ;
    private $idfaturaorca;
    private $cdmodpgto;
    private $qtmedidaorca;

    public function __construct($nrhierarquiaserv = null, $servicoPrincipal = null, $nrseqorcamento = null){
      // set_time_limit(900);
      $this->nrorg = \Geral\Util\Ambiente::getNrorg();
      $this->nrhierarquiaserv = $nrhierarquiaserv;
      $this->servicoPrincipal = $servicoPrincipal;
      $this->nrseqorcamento = $nrseqorcamento;
      if(!empty($nrseqorcamento)){
        $preco = OrcamentoLogic::retornaCdprecbaseOrca($nrseqorcamento, $this->nrorg);
        $this->cdprecobase = $preco['CDPRECBASESERV'];
        $this->modeloorca = $preco['NRORCAMENTOMODELO'];
        $this->itemespelho = OrcamentoLogic::retornaItemEspelho($this->modeloorca, $this->nrorg);
        $this->tipomedida = $this->itemespelho['CDTPMEDIDASERV'];
      }  
    }

    public function setNrorg($nrorg){
      $this->nrorg = $nrorg;
    }
    public function getNrorg(){
      return $this->nrorg;
    }
    public function setTipomedida($tipomedida){
      $this->tipomedida = $tipomedida;
    }
    public function getTipomedida(){
      return $this->tipomedida;
    }
    public function setModeloorca($modeloorca){
      $this->modeloorca = $modeloorca;
    }
    public function getModeloorca(){
      return $this->modeloorca;
    }
    public function setServicoPrincipal($servicoPrincipal){
      $this->servicoPrincipal = $servicoPrincipal;
    }
    public function getServicoPrincipal(){
      return $this->servicoPrincipal;
    }
    public function setNrseqorcamento($nrseqorcamento){
      $hits->nrseqorcamento = $nrseqorcamento;
    }
    public function getNrseqorcamento(){
      return $hits->nrseqorcamento;
    }
    public function setNrhierarquiaserv($nrhierarquiaserv){
      $hits->nrhierarquiaserv = $nrhierarquiaserv;
    } 
    public function getNrhierarquiaserv(){
      return $hits->nrhierarquiaserv;
    }
    
    /* Função para verificar o valor do parâmetro IDSOMAVALORSUP*/
    public function verifIdSomaValorSup($nrHierarquia = null, $servico=null, $nrorg=null){
      
   	  $sql = "SELECT IDSOMAVALORSUP
   	            FROM FCT_HIERARQUIASERV
   	           WHERE NRHIERARQUIASERV = :NRHIERARQUIASERV
   	             AND CDSERVORCAREL = :CDSERVORCAREL
   	             AND NRORG = :NRORG";
   	  $params = array('NRORG'=>$nrorg,
   	  	              'NRHIERARQUIASERV'=>$nrHierarquia,
   	  	              'CDSERVORCAREL'=>$servico);

   	  $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
   	  if(count($retorno)>0){
   	  	 return $retorno[0]['IDSOMAVALORSUP'];
   	  }else{
   	     return null;
   	  }
    }

    /* Função criada para verificar o valor do parametro IDHIERARQUIAOBRIG */
    public static function verifIdHierarquiaObrig($nrHierarquia, $nrorg){
      $sql = "SELECT IDHIERARQUIAOBRIG
   	            FROM FCT_HIERARQUIASERV
   	           WHERE NRHIERARQUIASERV = :NRHIERARQUIASERV
   	             AND NRORG = :NRORG";
   	  $params = array('NRORG'=>$nrorg,
   	  	              'NRHIERARQUIASERV'=>$nrHierarquia);

   	  $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
   	  if(count($retorno)>0){
   	  	 return $retorno[0]['IDHIERARQUIAOBRIG'];
   	  }else{
   	     return null;
   	  }
    }

    /*Função criada para retornar o tipo de hierarquia de um serviço */
    public static function verificaHierarquiaServ($servHierarq, $nrHierarquia, $nrorg){
      $sql = "SELECT CDSERVORCASUP 
               FROM FCT_HIERARQUIASERV
              WHERE ((NRTIPOHIERARQSERV = :NRTIPOHIERARQSERV) OR (:NRTIPOHIERARQSERV IS NULL))
                AND NRORG = :NRORG
                AND CDSERVORCAREL = :CDSERVORCAREL";

      $params = array("NRORG"=>$nrorg,
      	              "CDSERVORCAREL"=>$servHierarq,
      	              "NRTIPOHIERARQSERV"=>$nrHierarquia);

      $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql,$params);
      if(count($retorno)>0){
         if(!empty($retorno[0]['CDSERVORCASUP'])){
         	return self::TIPO_INFERIOR;
         }else{
         	return self::TIPO_PRINCIPAL;
         }
      }else{
         return self::TIPO_SEM_HIERARQUIA;
      }
    }
   
    public function calculaPrecoServico($servPrinc = null, $nrHierarquia = null){
      $precoServ = 0;
      if(empty($nrHierarquia)){
        $nrHierarquia = $this->nrhierarquiaserv;
      }

      if(empty($servPrinc)){
        $servPrinc = $this->servicoPrincipal;
      }

      $tipoHierarquia = 'INFERIOR';//self::verificaHierarquiaServ($servPrinc, $nrHierarquia, $this->nrorg); TODO
      if( $tipoHierarquia == self::TIPO_SEM_HIERARQUIA ){
          return self::TIPO_SEM_HIERARQUIA;


      }else{
          
      	  $servInferiores = $this->buscaServicosInferiores($servPrinc);

          if($servInferiores != self::STR_VAZIO){
      	     $precoServ = $this->percorreArrDependentes($servInferiores,$this->tipomedida);
          }else{
            
             $precoServ = OrcamentoLogic::buscaPrecoBase($servPrinc, $this->cdprecobase, $this->tipomedida, $this->nrorg);
          }
          
          return $precoServ;
      }

    }

    public function percorreArrDependentes($arrDepend, $tipomedida){
   	  $precoServ = 0;
      if($arrDepend != self::STR_VAZIO){
      	 foreach($arrDepend as $dependente){
           if(self::verifIdSomaValorSup($dependente['NRHIERARQUIASERV'], $dependente['CDSERVORCAREL'], $this->nrorg) == self::STR_SIM ){
	            if($dependente['ARR_DEPENDENTES'] == self::STR_VAZIO){
	                $precoServ += OrcamentoLogic::buscaPrecoBase($dependente['CDSERVORCAREL'], $this->cdprecobase, $tipomedida, $this->nrorg);	
	            }
              $precoServ += $this->percorreArrDependentes($dependente['ARR_DEPENDENTES'], $tipomedida);
           }
	         

         }
      }
      return $precoServ;
    }

    public function buscaServicosInferiores($servSuperior = null, $nivel = 1, $cdhierarquia = null, $incCdhierarquia = true, $arvore = null){
      try{

	   	    $sql = "SELECT CDSERVORCAREL, NMSERVORCA, NRTIPOHIERARQSERV, CDSERVORCASUP, NRHIERARQUIASERV
                   FROM FCT_HIERARQUIASERV, FCT_SERVICOORCA
                  WHERE ((FCT_HIERARQUIASERV.CDSERVORCASUP    = :CDSERVORCASUP) OR (CDSERVORCASUP IS NULL AND :CDSERVORCASUP IS NULL))
                    AND FCT_HIERARQUIASERV.NRORG            = :NRORG
                    AND FCT_HIERARQUIASERV.CDSERVORCAREL = FCT_SERVICOORCA.CDSERVORCA
                    AND FCT_HIERARQUIASERV.NRORG = FCT_SERVICOORCA.NRORG
                  ORDER BY NRHIERARQUIASERV";

		      $params = array("NRORG"=>$this->nrorg, "CDSERVORCASUP"=>"$servSuperior");
    		  $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);

		      if(count($retorno)>0){
		  	     if($incCdhierarquia){
                  $cdhierarquia = 0;   
             }
             
		  	     for($i=0; $i<=count($retorno)-1; $i++){

		  	 	     if($incCdhierarquia){
                  $cdhierarquia++;   
                  
               }
               if($arvore){
                  $nv = $i+1;
                  $cdarv = $arvore."."."$nv";

               }else{
                  $cdarv = $i+1;
               }
               $retorno[$i]['ARVORE'] = "$cdarv";
               
               $retorno[$i]['NIVEL'] = $nivel;
               $retorno[$i]['NRSEQHIERARQUIA'] = $cdhierarquia;
               
	             $retorno[$i]['ARR_DEPENDENTES'] = $this->buscaServicosInferiores($retorno[$i]['CDSERVORCAREL'], $nivel+1,$cdhierarquia,false, $retorno[$i]['ARVORE']);
	           }       
	           return $retorno;
		      }else{
		  	     return self::STR_VAZIO;
		      }
	    }catch(\Exception $e){
		     throw $e;
	    }
    }

    public function listaServHierarquia(){
       $sql = 'DELETE FROM TRAB_SERVHIERARQUIA 
                  WHERE NRORG = :NRORG';
        
       $params = array('NRORG'=>$this->nrorg);
       $retorno = @\Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
       $servicosInferiores = $this->buscaServicosInferiores();
       $this->insereDependentes($servicosInferiores);
    }   

    public function insereDependentes($arrDepend){  
      if($arrDepend != self::STR_VAZIO){
      	 foreach($arrDepend as $dependente){
           $params['NRSEQTRABSERV'] = \Geral\Util\NovoCodigo::geraCodigo($this->nrorg,'NRSEQTRABSERV', 1,10);
      	 	 $params['CDSERVORCAREL'] = $dependente['CDSERVORCAREL'];
           $params['CDSERVORCASUP'] = $dependente['CDSERVORCASUP'];
           $params['NRNIVELHIERARQUIA'] = $dependente['NIVEL'];
           $params['NRSEQHIERARQUIA'] = $dependente['NRSEQHIERARQUIA'];
           $params['NMSERVORCA'] = $dependente['NMSERVORCA'];
           $params['NRTIPOHIERARQSERV'] = $dependente['NRTIPOHIERARQSERV'];
           $params['CDARVORE'] = $dependente['ARVORE'];
           $params['NRORG'] = $this->getNrorg();
           $params['CDOPERINCLUSAO'] = \Geral\Util\Ambiente::getCdoperador(); 

           self::insertTrabServ($params);

	         $this->insereDependentes($dependente['ARR_DEPENDENTES']);
        }
      }
      return true;
    }

    public static function insertTrabServ($params){
      try{
          $sql = 'INSERT INTO TRAB_SERVHIERARQUIA (NRSEQTRABSERV, NMSERVORCA, CDSERVORCAREL, CDSERVORCASUP,  NRSEQHIERARQUIA, NRTIPOHIERARQSERV, 
      	                                            NRNIVELHIERARQUIA, NRORG, CDOPERINCLUSAO, DTINCLUSAO, CDARVORE)
										                    VALUES (:NRSEQTRABSERV,:NMSERVORCA, :CDSERVORCAREL, :CDSERVORCASUP, :NRSEQHIERARQUIA, :NRTIPOHIERARQSERV,
                                                   :NRNIVELHIERARQUIA, :NRORG, :CDOPERINCLUSAO, SYSDATE, :CDARVORE)';

          //\Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
          @\Setforms2\Runtime\Connection::getInstance()->execute($sql, $params);
          return true;
      }catch(\Exception $e){
      	 throw $e;
      }
    }

    public static function listaServHierarquia_Wiz($nrorg, $nrorcamentomodelo){
   	  $sql = 'SELECT IT.CDSERVORCA 
                FROM FCT_ITEMORCAMODELO IT, FCT_HIERARQUIASERV HI
               WHERE IT.CDSERVORCA = HI.CDSERVORCAREL
                 AND HI.CDSERVORCASUP IS NULL
                 AND IT.NRTIPOHIERARQSERV = HI.NRTIPOHIERARQSERV
                 AND IT.NRORG = HI.NRORG
                 AND IT.NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                 AND IT.NRORG = :NRORG';

      $params = array("NRORG"=>$nrorg, "NRORCAMENTOMODELO"=>$nrorcamentomodelo);
   	  $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);

   	  if(count($retorno) >0){
         $strIn = '';
         foreach($retorno as $servico){
            $strIn .= "'".$servico['CDSERVORCA']."', ";
         }
         $strIn = " FCT_ITEMORCAMENTO.CDSERVORCA IN (".substr($strIn,0,-2).")";
         return $strIn;
   	  }else{
         return ' 1 = 1 ';
      }
    }
   
    public function insereServHierarquia($servInsert, $nrHierarquia = null, $nmservorca = null, $arvore = null){
      try{
         
         // $tipoHierarquia = self::verificaHierarquiaServ($servInsert, $nrHierarquia, $this->nrorg);
          $tipoHierarquia = 'INFERIOR';
         // $servInsert = \Facilities\Entities\FctServicoorca::getObjectsByWhere('NRORG = '.$this->nrorg.' AND CDSERVORCA = '.$servInsert);
         // $servInsert = $servInsert[0];
          if($tipoHierarquia == self::TIPO_SEM_HIERARQUIA){
            return self::TIPO_SEM_HIERARQUIA;
          }else{
            //$itemExiste = \Services\Logic\OrcamentoLogic::verificaItemOrcamento($this->nrorg, $this->nrseqorcamento, $servInsert);
            //\Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
            //if($itemExiste[0]['QUANTIDADE'] == 0){
            if(0 == 0){
               /* $sql = 'SELECT IT.CDTPMEDIDASERV, IT.NRGRUPOFATURA, IT.IDFATURAORCA, IT.CDMODPGTO
                          FROM FCT_ITEMORCAMENTO IT, FCT_HIERARQUIASERV HIE
                         WHERE IT.CDSERVORCA = HIE.CDSERVORCASUP
                           AND IT.NRSEQORCAMENTO = :NRSEQORCAMENTO
                           AND HIE.CDSERVORCAREL = :CDSERVORCAREL';    
               $params = array("NRSEQORCAMENTO"=>$this->nrseqorcamento, 
                               "CDSERVORCAREL"=>$servInsert);
               $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql,$params);
 */
              /* $grupofatura = '';
               $idfaturaorca = '';
               $cdmodpgto = '';
               //$this->tipomedida = 5;
               if(count($retorno)>0){
                  $this->tipomedida = $retorno[0]['CDTPMEDIDASERV'];
                  $grupofatura = $retorno[0]['NRGRUPOFATURA'];
                  $idfaturaorca = $retorno[0]['IDFATURAORCA'];
                  $cdmodpgto = $retorno[0]['CDMODPGTO'];
                }else{

                   $sql = 'SELECT CDTPMEDIDASERV,
                                 NRGRUPOFATURA,
                                 IDFATURAORCA,
                                 CDMODPGTO
                            FROM FCT_ITEMORCAMODELO
                           WHERE NRORG = :NRORG
                             AND NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                             AND IDITEMESPELHO = \'SIM\'
                             AND IDADICITEMORC = \'N\'';

                   $params = array('NRORG'=>$this->nrorg, 'NRORCAMENTOMODELO'=>$this->modeloorca);
                   $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
                   if(count($retorno)>0){
                      $this->tipomedida = $retorno[0]['CDTPMEDIDASERV'];
                      $grupofatura = $retorno[0]['NRGRUPOFATURA'];
                      $idfaturaorca = $retorno[0]['IDFATURAORCA'];
                      $cdmodpgto = $retorno[0]['CDMODPGTO'];
                   }

                }
                */
               $preco =  self::retornaPrecoHierarquia($arvore, $this->cdprecobase, $this->nrorg);
                //   $preco =  $this->calculaPrecoServico($servInsert, $nrHierarquia);
           /* todo:Comentar o codigo a baixo 
               $itModelo = self::getItemModelo($this->modeloorca,$servInsert, $this->nrorg);
               if(count($itModelo)>0){
                  $this->grupofatura = $itModelo[0]['NRGRUPOFATURA'];
                  $this->idfaturaorca = $itModelo[0]['IDFATURAORCA'];
                  $this->cdmodpgto = $itModelo[0]['CDMODPGTO'];
                  $this->tipomedida = $itModelo[0]['CDTPMEDIDASERV'];
               }else{
                  $this->grupofatura = $this->itemespelho['NRGRUPOFATURA'];
                  $this->idfaturaorca = $this->itemespelho['IDFATURAORCA'];
                  $this->cdmodpgto = $this->itemespelho['CDMODPGTO'];
                  $this->tipomedida = $this->itemespelho['CDTPMEDIDASERV'];
               }

          /*todo: comentar o codigo acima*/
               $paramItemOrcamento = array("NRORG"=>$this->nrorg,
                                           "NRITEMORCAMENTO"=>\Geral\Util\NovoCodigo::geraCodigo($this->nrorg,'NRITEMORCAMENTO',1,10),
                                           "NRSEQORCAMENTO"=>$this->nrseqorcamento,
                                           "CDOPERINCLUSAO"=>\Geral\Util\Ambiente::getCdoperador(),
                                           "DTINCLUSAO"=>now(),
                                           "CDSERVORCA"=>$servInsert,
                                           "CDPRODUTO"=>"",
                                           "NRGRUPOFATURA"=>$this->grupofatura,
                                           "QTITEMORCAMENTO"=>'1',
                                           "CDTPMEDIDASERV"=>$this->tipomedida,
                                           "QTMEDIDAORCA"=>$this->qtmedidaorca,
                                           "IDFATURAORCA"=>$this->idfaturaorca,
                                           "CDMODPGTO"=>$this->cdmodpgto,
                                           "VRUNITITEMORCA"=>$preco,
                                           "VRTOTALITEMORCA"=>$preco * $this->qtmedidaorca,
                                           "VRITEMSEMDESC"=>$preco * $this->qtmedidaorca,
                                           "NMITEMORCAMENTO"=>$nmservorca,
                                           "IDITEMOBRIGORCA"=>'N',
                                           "IDIMPRESSAORCA"=>'S',
                                           "IDATIVO"=>"S",
                                           "NRITEMORCAMENTOPAI"=>"",
                                           "SGUNIDADE"=>'UN',
                                           "IDIMPRESSAOORCA"=>'S',
                                           "IDSOMASUBITENS"=>'S',
                                           "IDSOMANOITEMPAI"=>'S',
                                           "IDSOMANOTOTALORCA"=>'S',
                                           "IDTRANSPORTADORA"=>"TERCEIRO",
                                           "DSQTITEMORCA"=>"Referente ao Serviço",
                                           "IDBLOQALTORC"=>'N',
                                           "IDADICITEMORC"=>'N',
                                           "IDADICCOBRITEMORC"=>'N',
                                           "IDMULTMEDVRTOT"=>'S',
                                           "DTINICFATORC"=>"",
                                           "NRTIPOHIERARQSERV" => $nrHierarquia
                                           ); 
                

                @OrcamentoLogic::setaItemOrcamento($paramItemOrcamento);
                
            }else{
              $preco = $this->calculaPrecoServico($servInsert, $nrHierarquia);
          //    $item = @OrcamentoLogic::getItemOrcaProd($this->nrseqorcamento, $servInsert, null,$this->nrorg);
              
              @OrcamentoLogic::updateVrOrcamento($this->nrorg, $this->nrseqorcamento, $servInsert, null, $preco);
            }
            @OrcamentoLogic::verificaServDependentes($this->nrseqorcamento, $servInsert, 'S', $this->nrorg);
            return true;
          }
      }catch(\Exception $e){
        throw $e;
      } 
   }
   
   public function insereServHierarquia_wiz(array $listaServ){
    // set_time_limit(900);
     // $servicos = \Facilities\Entities\FctItemorcamento::getObjectsByWhere("NRORG = $this->nrorg AND NRSEQORCAMENTO = $this->nrseqorcamento");
      foreach($listaServ as $servico){
        //$this->insereServHierarquia($servico);
        $itemProd = self::getItemOrcaProd($this->nrseqorcamento, $servico, null, $this->nrorg);
        if(count($itemProd)>0){
           $this->grupofatura = $itemProd[0]['NRGRUPOFATURA'];
           $this->idfaturaorca = $itemProd[0]['IDFATURAORCA'];
           $this->cdmodpgto = $itemProd[0]['CDMODPGTO'];
           $this->tipomedida = $itemProd[0]['CDTPMEDIDASERV'];
           $this->qtmedidaorca = $itemProd[0]['QTMEDIDAORCA'];
        }
        $inferiores = $this->buscaServicosInferioresNoItem($servico);
      
        $this->insereItensInferiores($inferiores);
      }
      return true;
   }

    public function insereServHierarquiaItem(array $listaServ){
      foreach($listaServ as $servico){
       // $this->insereServHierarquia($servico['CDSERVORCA'], $servico['NRTIPOHIERARQSERV']);
       //$count++;

        $itemOrcaPai = self::getItemPaiHierarquia($this->nrorg,$this->nrseqorcamento,$servico['CDARVORE']);
        if (count($itemOrcaPai)>0){
          $this->grupofatura = $itemOrcaPai[0]['NRGRUPOFATURA'];
          $this->idfaturaorca = $itemOrcaPai[0]['IDFATURAORCA'];
          $this->cdmodpgto = $itemOrcaPai[0]['CDMODPGTO'];
          $this->tipomedida = $itemOrcaPai[0]['CDTPMEDIDASERV'];
          $this->qtmedidaorca = $itemOrcaPai[0]['QTMEDIDAORCA'];
        }else{
          $itModelo = self::getItemModelo($this->modeloorca,$servico['CDSERVORCA'], $this->nrorg);
          if(count($itModelo)>0){
            $this->grupofatura = $itModelo[0]['NRGRUPOFATURA'];
            $this->idfaturaorca = $itModelo[0]['IDFATURAORCA'];
            $this->cdmodpgto = $itModelo[0]['CDMODPGTO'];
            $this->tipomedida = $itModelo[0]['CDTPMEDIDASERV'];
            $this->qtmedidaorca = $itModelo[0]['QTMEDIDAORCA'];
          }else{
            $this->grupofatura = $this->itemespelho['NRGRUPOFATURA'];
            $this->idfaturaorca = $this->itemespelho['IDFATURAORCA'];
            $this->cdmodpgto = $this->itemespelho['CDMODPGTO'];
            $this->tipomedida = $this->itemespelho['CDTPMEDIDASERV'];
            $this->qtmedidaorca = $this->itemespelho['QTMEDIDAORCA'];
            }
        }
        
        $inferiores = $this->buscaServicosInferioresNoItem($servico['CDARVORE']);
        $this->insereItensInferiores($inferiores);
      }
      return true;
    }

    public function insereItensInferiores($arrDepend){
      if($arrDepend != self::STR_VAZIO){
        foreach($arrDepend as $dependente){
          $this->insereServHierarquia($dependente['CDSERVORCAREL'], $dependente['NRTIPOHIERARQSERV'], $dependente['NMSERVORCA'], $dependente['CDARVORE']);
         //  $this->insereItensInferiores($dependente['ARR_DEPENDENTES']);
        }
      }
      return true;
    }

    public static function insereItemPaiHierarquia($nrorg, $nrseqorcamento, $cdservorca, $nrtipohierarquia, $valor, $valoruni){
      
      $select = "SELECT NRORCAMENTOMODELO FROM FCT_ORCAMENTO WHERE NRSEQORCAMENTO = ".$nrseqorcamento;
      $nrmodelo = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($select);
      $selectServ = "SELECT CDSERVORCA FROM FCT_ITEMORCAMODELO WHERE NRORCAMENTOMODELO = ".$nrmodelo[0]['NRORCAMENTOMODELO']." AND NRTIPOHIERARQSERV = ".$nrtipohierarquia;
      $lastService = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($selectServ);
      if(empty($lastService))
      {
        $lastService = 0;
      }
      if($cdservorca != $lastService[0]['CDSERVORCA']){
        $sql = "SELECT FCT_HIERARQUIASERV.CDSERVORCAREL, FCT_HIERARQUIASERV.NRTIPOHIERARQSERV, FCT_HIERARQUIASERV.CDSERVORCASUP,FCT_SERVICOORCA.NMSERVORCA, FCT_ITEMORCAMENTO.NRSEQORCAMENTO, FCT_ITEMORCAMENTO.CDTPMEDIDASERV, FCT_ITEMORCAMENTO.IDFATURAORCA,FCT_ITEMORCAMENTO.NRGRUPOFATURA, FCT_ITEMORCAMENTO.CDMODPGTO
                  FROM FCT_HIERARQUIASERV, FCT_SERVICOORCA, FCT_ITEMORCAMENTO
                 WHERE ((FCT_HIERARQUIASERV.CDSERVORCAREL    = :CDSERVORCASUP))
                   AND FCT_HIERARQUIASERV.NRORG            = :NRORG
                   AND FCT_SERVICOORCA.CDSERVORCA = FCT_HIERARQUIASERV.CDSERVORCASUP
                   AND FCT_SERVICOORCA.NRORG = FCT_HIERARQUIASERV.NRORG
                   AND FCT_ITEMORCAMENTO.CDSERVORCA = FCT_HIERARQUIASERV.CDSERVORCAREL
                   AND FCT_ITEMORCAMENTO.NRORG = FCT_HIERARQUIASERV.NRORG
                   AND FCT_ITEMORCAMENTO.NRSEQORCAMENTO = :NRSEQORCAMENTO
                 ORDER BY FCT_HIERARQUIASERV.NRHIERARQUIASERV";

        $params  = array("NRORG"=>$nrorg, "NRSEQORCAMENTO"=>$nrseqorcamento, "CDSERVORCASUP"=>"$cdservorca", 'NRTIPOHIERARQSERV'=>$nrtipohierarquia);
        $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
        if(count($retorno)>0){
          for($i=0; $i<=count($retorno)-1; $i++){
            $itemExiste = \Services\Logic\OrcamentoLogic::verificaItemOrcamento($nrorg, $nrseqorcamento, $retorno[$i]['CDSERVORCASUP']);
            if($itemExiste[0]['QUANTIDADE'] == 0){
              self::insereItemPai($nrorg, $nrseqorcamento, $retorno[$i]['CDSERVORCASUP'], $retorno[$i]['NRGRUPOFATURA'], $retorno[$i]['CDTPMEDIDASERV'], $retorno[$i]['IDFATURAORCA'], $retorno[$i]['CDMODPGTO'], $valor, $retorno[$i]['NMSERVORCA'], $retorno[$i]['NRTIPOHIERARQSERV']);
            }else{
              \Services\Logic\OrcamentoLogic::atualizaPrecoOrcamento($nrorg, $nrseqorcamento, $retorno[$i]['CDSERVORCAREL'],$nrtipohierarquia, $valor,$valoruni, false); 
            }
            self::insereItemPaiHierarquia($nrorg, $nrseqorcamento, $retorno[$i]['CDSERVORCASUP'], $retorno[$i]['NRTIPOHIERARQSERV'], $valor,$valoruni);  
            }
        }else{
          return 0;
        }
      }else{
          return 0;
      }
    }

    public static function insereItemPai($nrorg, $nrseqorcamento, $cdservorca, $grupofatura, $tipomedida, $idfaturaorca, $cdmodpgto, $preco, $nmservorca,$hierarquia){
      $paramItemOrcamento = array("NRORG"=>$nrorg,
                                  "NRITEMORCAMENTO"=>\Geral\Util\NovoCodigo::geraCodigo($nrorg,'NRITEMORCAMENTO',1,10),
                                  "NRSEQORCAMENTO"=>$nrseqorcamento,
                                  "CDOPERINCLUSAO"=>'1',
                                  "DTINCLUSAO"=>now(),
                                  "CDSERVORCA"=>$cdservorca,
                                  "CDPRODUTO"=>"",
                                  "NRGRUPOFATURA"=>$grupofatura,
                                  "QTITEMORCAMENTO"=>'1',
                                  "CDTPMEDIDASERV"=>$tipomedida,
                                  "QTMEDIDAORCA"=>'1',
                                  "IDFATURAORCA"=>$idfaturaorca,
                                  "CDMODPGTO"=>$cdmodpgto,
                                  "VRUNITITEMORCA"=>$preco,
                                  "VRTOTALITEMORCA"=>$preco,
                                  "VRITEMSEMDESC"=>$preco,
                                  "NMITEMORCAMENTO"=>$nmservorca,
                                  "IDITEMOBRIGORCA"=>'N',
                                  "IDIMPRESSAORCA"=>'S',
                                  "IDATIVO"=>"S",
                                  "NRITEMORCAMENTOPAI"=>"",
                                  "SGUNIDADE"=>'UN',
                                  "IDIMPRESSAOORCA"=>'S',
                                  "IDSOMASUBITENS"=>'S',
                                  "IDSOMANOITEMPAI"=>'S',
                                  "IDSOMANOTOTALORCA"=>'S',
                                  "IDTRANSPORTADORA"=>"TERCEIRO",
                                  "DSQTITEMORCA"=>"Referente ao Serviço",
                                  "IDBLOQALTORC"=>'N',
                                  "IDADICITEMORC"=>'N',
                                  "IDADICCOBRITEMORC"=>'N',
                                  "IDMULTMEDVRTOT"=>'S',
                                  "DTINICFATORC"=>"",
                                  "NRTIPOHIERARQSERV" => $hierarquia
                                  ); 
      @OrcamentoLogic::setaItemOrcamento($paramItemOrcamento);
    }
    

    public function buscaServicosInferioresNoItem($servSuperior = null, $nivel = 1, $cdhierarquia = null, $incCdhierarquia = true, $arvore = null){
      try{
        $sql = "SELECT T.CDSERVORCAREL, T.NMSERVORCA, T.NRTIPOHIERARQSERV, T.CDSERVORCASUP, T.CDARVORE
                  FROM TRAB_SERVHIERARQUIA T
                 WHERE T.CDARVORE||'.' BETWEEN CONCAT(:CDARVORE,'.') AND RPAD(CONCAT(:CDARVORE,'.'),200,'Z')
                   AND T.NRORG = :NRORG
                   AND NOT EXISTS (SELECT 1 FROM FCT_ITEMORCAMENTO IT
                                    WHERE T.CDSERVORCAREL = IT.CDSERVORCA
                                      AND IT.NRORG = :NRORG
                                      AND IT.NRSEQORCAMENTO = :NRSEQORCAMENTO) 
                                    ORDER BY NRSEQTRABSERV";

        $params = array("NRORG"=>$this->nrorg, "CDARVORE"=>"$servSuperior", "NRSEQORCAMENTO"=>$this->nrseqorcamento);
        $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
        
        if(count($retorno)>0){
          return $retorno;
        }else{
          return self::STR_VAZIO;
        }
      }catch(\Exception $e){
        throw $e;
      }
    }
   
    public static function retornaPrecoHierarquia($cdarvore, $cdprecobase, $nrorg){
      $sql = "SELECT NVL(SUM(VRPRECO),0) AS VRPRECO
                FROM TRAB_SERVHIERARQUIA TR1, FCT_ITPRECBASESERV PR
               WHERE TR1.CDARVORE||'.' BETWEEN CONCAT(:CDARVORE,'.') AND RPAD(CONCAT(:CDARVORE,'.'),200,'Z') 
                 AND TR1.CDSERVORCAREL = PR.CDSERVORCA
                 AND TR1.NRORG = PR.NRORG
                 AND SYSDATE BETWEEN DTINIPBSERV AND NVL(DTFIMVALPBSERV, SYSDATE)
                 AND CDPRECBASESERV = :CDPRECBASESERV
                 AND NOT EXISTS (SELECT NRSEQTRABSERV 
                                   FROM TRAB_SERVHIERARQUIA TR2 
                                  WHERE TR2.CDSERVORCASUP =  TR1.CDSERVORCAREL
                                    AND TR2.NRORG = TR1.NRORG)";

      $params = array("NRORG"=>$nrorg, "CDPRECBASESERV"=>$cdprecobase, "CDARVORE"=>$cdarvore);
      $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
      return $retorno[0]['VRPRECO']; 
    }

    public static function retornaCdArvore($nrorg, $cdservorca){
      $sql = "SELECT CDARVORE FROM TRAB_SERVHIERARQUIA WHERE NRORG = :NRORG AND CDSERVORCAREL = :CDSERVORCAREL";
      $params = array("NRORG"=>$nrorg, "CDSERVORCAREL"=>$cdservorca);
      $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
      if(count($retorno)>0){
        return $retorno[0]['CDARVORE'];
      }
    }

    public static function getItemModelo($nrorcamentomodelo, $cdservorca, $nrorg){
      $sql = "SELECT CDTPMEDIDASERV,  NRGRUPOFATURA, IDFATURAORCA, CDMODPGTO, QTMEDIDAORCA 
                FROM FCT_ITEMORCAMODELO 
               WHERE NRORCAMENTOMODELO = :NRORCAMENTOMODELO
                 AND NRORG = :NRORG
                 AND CDSERVORCA = :CDSERVORCA";
                 

      $params = array("NRORG"=>$nrorg,"NRORCAMENTOMODELO"=>$nrorcamentomodelo, "CDSERVORCA"=>$cdservorca);
      $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql, $params);
      return $retorno;
    }
    
    public static function getItemOrcaProd($nrOrcamento, $servOrca, $tipoMedida = null, $nrorg){
      try{
         $sql = "SELECT IT.NRITEMORCAMENTO, IT.CDTPMEDIDASERV, IT.QTMEDIDAORCA,  IT.NRGRUPOFATURA, IT.IDFATURAORCA, IT.CDMODPGTO
                   FROM FCT_ITEMORCAMENTO IT, TRAB_SERVHIERARQUIA TR
                  WHERE IT.NRSEQORCAMENTO = :NRSEQORCAMENTO
                    AND TR.CDARVORE = :CDSERVORCA
                    AND TR.CDSERVORCAREL = IT.CDSERVORCA
                    AND ((IT.CDTPMEDIDASERV = :CDTPMEDIDASERV) OR (:CDTPMEDIDASERV IS NULL))
                    AND IT.NRORG = :NRORG";

         $params = array('NRSEQORCAMENTO'=>$nrOrcamento,
                         'CDSERVORCA'=>$servOrca,
                         'CDTPMEDIDASERV'=>$tipoMedida,
                         'NRORG'=>$nrorg);
         

         $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql,$params);
         
            return $retorno;
         
      }catch(\Exception $e){
         throw $e;
      }
    }
   
    public static function getItemPaiHierarquia($nrorg, $nrseqorcamento, $cdarvore){
      $sql = "SELECT IT.NRITEMORCAMENTO, IT.CDTPMEDIDASERV, IT.QTMEDIDAORCA,  IT.NRGRUPOFATURA, IT.IDFATURAORCA, IT.CDMODPGTO
                FROM FCT_ITEMORCAMENTO IT, TRAB_SERVHIERARQUIA TR
               WHERE IT.NRSEQORCAMENTO = :NRSEQORCAMENTO
                 AND TR.CDSERVORCAREL = IT.CDSERVORCA
                 AND TR.CDARVORE = SUBSTR(:CDARVORE,0,INSTR(:CDARVORE,'.')-1)
                 AND TR.NRORG = :NRORG
                 AND IT.NRORG = :NRORG";
      $params = array("NRORG"=>$nrorg,"CDARVORE"=>$cdarvore,"NRSEQORCAMENTO"=>$nrseqorcamento);
      $retorno = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql,$params);
      return $retorno;
    }
}