/* Sistemas Integrados TECNISA
   Servios

Nome: AJUSTA_EXPEDIENTE

Descrio:
Retorna uma data/hora ajustando-a para o prximo horrio de expediente vlido.

P_CDFILIAL   - Cdigo da Filial
P_CDOPERRESP - Cdigo do OPERADOR
P_DTHRREFE   - Data/hora de referncia
P_IDAJUSTE   - Identificador do tipo de ajuste ('CONSIDERA_LIMITE', 'NAO_CONSIDERA_LIMITE')

Para aplicar e debugar
-- @H:\SISTEMAS\DBA\SQL\ORACLE\7198_SRVFUN_AJUSTA_EXPEDIENTE.SQL
-- AJUSTA_EXPEDIENTE('9999', '01', '0000000367', TO_DATE('19991213 0759', 'YYYYMMDD HH24MI'))
-- SET SERVEROUTPUT ON
-- DBMS_OUTPUT.PUT_LINE('xx');

Criao:
13/12/99 - Ederson

*/

CREATE OR REPLACE
FUNCTION AJUSTA_EXPEDIENTE
  (P_CDFILIAL   IN VARCHAR2,
   P_CDOPERRESP IN VARCHAR2,
   P_DTHRREFE   IN DATE,
   P_IDAJUSTE   IN VARCHAR2) RETURN DATE IS

   V_DTHRENTR1  DATE;
   V_DTHRSAID1  DATE;
   V_DTHRENTR2  DATE;
   V_DTHRSAID2  DATE;

   V_DTHRRESULT DATE;

   V_QTSOBRA    NUMBER;

   V_ACHOU      NUMBER(1);

BEGIN

   /* - Verifica o expediente do dia passado como parmetro ----------------- */
   IF P_IDAJUSTE = 'CONSIDERA_LIMITE' THEN
      V_DTHRENTR1 := TO_DATE(TO_CHAR(P_DTHRREFE, 'YYYYMMDD ')||'0800', 'YYYYMMDD HH24MI');
      V_DTHRSAID1 := TO_DATE(TO_CHAR(P_DTHRREFE, 'YYYYMMDD ')||'1200', 'YYYYMMDD HH24MI');
      V_DTHRENTR2 := TO_DATE(TO_CHAR(P_DTHRREFE, 'YYYYMMDD ')||'1321', 'YYYYMMDD HH24MI');
      V_DTHRSAID2 := TO_DATE(TO_CHAR(P_DTHRREFE, 'YYYYMMDD ')||'1800', 'YYYYMMDD HH24MI');
      /* -------------------------------------------------------------------- */
   END IF;
   IF P_IDAJUSTE = 'NAO_CONSIDERA_LIMITE' THEN
      V_DTHRENTR1 := TO_DATE(TO_CHAR(P_DTHRREFE, 'YYYYMMDD ')||'0800', 'YYYYMMDD HH24MI');
      V_DTHRSAID1 := TO_DATE(TO_CHAR(P_DTHRREFE, 'YYYYMMDD ')||'1201', 'YYYYMMDD HH24MI');
      V_DTHRENTR2 := TO_DATE(TO_CHAR(P_DTHRREFE, 'YYYYMMDD ')||'1321', 'YYYYMMDD HH24MI');
      V_DTHRSAID2 := TO_DATE(TO_CHAR(P_DTHRREFE, 'YYYYMMDD ')||'1801', 'YYYYMMDD HH24MI');
   END IF;
   /* ----------------------------------------------------------------------- */

   /* Utilizado para somente empurrar a data/hora para o prximo expediente
      sem somar possveis sobras de tempo aps o expediente */
   IF (P_DTHRREFE >= V_DTHRENTR1 AND P_DTHRREFE < V_DTHRSAID1) OR
      (P_DTHRREFE >= V_DTHRENTR2 AND P_DTHRREFE < V_DTHRSAID2) THEN
      V_DTHRRESULT := P_DTHRREFE;
   ELSE
      IF P_DTHRREFE <= V_DTHRENTR2 THEN
         V_DTHRRESULT := V_DTHRENTR2;
      END IF;
      IF P_DTHRREFE <= V_DTHRENTR1 THEN
         V_DTHRRESULT := V_DTHRENTR1;
      END IF;
      IF (P_DTHRREFE >= V_DTHRSAID2) THEN
         V_DTHRRESULT := TO_DATE(TO_CHAR(P_DTHRREFE, 'YYYYMMDD ')||'0800', 'YYYYMMDD HH24MI') + 1;
      END IF;
   END IF;

   /* Retorna a data/hora no prximo dia vlido (verifica se no  feriado) */
   LOOP
      SELECT
         COUNT(*)
      INTO
         V_ACHOU
      FROM
         FERIFILI
      WHERE
         (CDFILIAL   = P_CDFILIAL) AND
         (DTFERIFILI = TO_DATE(TO_CHAR(V_DTHRRESULT, 'YYYYMMDD'), 'YYYYMMDD'));
      IF V_ACHOU = 1 THEN
         /* Verifica se  feriado */
         V_DTHRRESULT := TO_DATE(TO_CHAR(V_DTHRRESULT, 'YYYYMMDD ')||'0800', 'YYYYMMDD HH24MI') + 1;
      ELSE
         EXIT;
      END IF;
   END LOOP;

   RETURN(V_DTHRRESULT);
END;
/
