package com.javaseleniumtemplate.pages;

import com.javaseleniumtemplate.bases.PageBase;
import org.openqa.selenium.By;
import org.openqa.selenium.Keys;
import org.openqa.selenium.interactions.Actions;
import org.testng.Assert;

public class CategoriaEmbalagemPage extends PageBase {

    By botaoMenu = By.xpath("//div[@class='bars']");
    By campoPesquisar = By.xpath("//section[@class='zh-container-search-menu']//input[@type='text']");
    By opcaoTelaCategoriaEmbalagens = By.xpath("//span[text() = 'Categoria de Embalagens']");

    By botaoAdicionar = By.xpath("//div[@class=\"zh-footer-center\"]");
    By botaoSalvar = By.xpath("//div[@class=\"zh-footer-right\"]");
    By botaoCancelar = By.xpath("//div[@class=\"zh-footer-left\"]");
    By botaoExcluir = By.xpath("//div[@class=\"zh-footer-center\"]");
    By botaoAbrirOpcoesControleTelaInicial = By.xpath("//div[@data-ng-if=\"widget.isFloatingControlVisible()\"]//div[@data-zh-floating-control]");
    By botaoPesquisarFiltro = By.xpath("//div[@class=\"control-menu\"]//li[@data-zh-id=\"zh-floating-control-custom-action-1\"]");
    By campoProcurar = By.xpath("//li[@class=\"float-action expandable fix-position-bottom opened\"]//input");

    By campoCodigo = By.id("CDCATEMBALAG");
    By campoNome = By.id("NMCATEMBALAG");
    By mensagemObrigatorioCodigo = By.xpath("//div[@data-zh-id=\"932039233211315236995\"]//div[@class=\"zh-validation\"]/span");
    By mensagemObrigatorioNome = By.xpath("//div[@data-zh-id=\"93203923317775255997\"]//div[@class=\"zh-validation\"]/span");

    By checkBoxCategoriaLinha1 = By.xpath("//div[@id=\"grid-9320392325592474642427\"]//div[@class=\"body-content\"]/div[1]/div[1]/span");
    By codigoCategoriaLinha1 = By.xpath("//div[@id=\"grid-9320392325592474642427\"]//div[@class=\"body-content\"]/div[1]/div[2]/span");
    By nomeCategoriaLinha1 = By.xpath("//div[@id=\"grid-9320392325592474642427\"]//div[@class=\"body-content\"]/div[1]/div[3]/span");

    By mensagemCantoSuperiorDireito = By.xpath("//*[@class=\"zh-notification-messages\"]//span[@class=\"notification-message-text\"]");
    By mensagemAlerta = By.xpath("//section[@class=\"zh-container-alert\"]//div[@class=\"zh-alert-scroll\"]/p");
    By botaoSim = By.xpath("//section[@class=\"zh-container-alert\"]//button[@data-zh-id=\"zh-footer-alert-button-0\"]");
    By botaoOkAlerta = By.xpath("//section[@class=\"zh-container-alert\"]//button[@class=\"zh-alert-button\"]");

    public void acessarTela() {
        tempoParaAguardarTelaInicial();
        waitForElement(botaoMenu);
        click(botaoMenu);
        sendKeys(campoPesquisar, "Categoria de Embalagens");
        click(opcaoTelaCategoriaEmbalagens);
    }

    public void adicionarCategoria() {
        tempo();
        click(botaoAdicionar);
        sendKeys(campoCodigo, "5");
        sendKeys(campoNome, "QUINTERNÁRIA");
        click(botaoSalvar);

        preencherCampoProcurarTelaInicial("00005");
        //dados encontrados da categoria adicionada
        String codigo = getText(codigoCategoriaLinha1);
        String nome = getText(nomeCategoriaLinha1);

        Assert.assertEquals(codigo, "00005", "Verificação do código da categoria adicionada");
        Assert.assertEquals(nome, "QUINTERNÁRIA", "Verificação do nome da categoria adicionada");
    }

    public void tentarAdicionarCodigoExistente() {
        tempo();
        tempo();
        click(botaoAdicionar);
        tempo();
        sendKeys(campoCodigo, "3");
        sendKeys(campoNome, "TERCEIRA");
        click(botaoSalvar);

        //mensagem alertando que o registro já existe
        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Registro já existe.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar se após tentar adicionar uma categoria com código já existente, é exibida uma mensagem na tela informando que o registro já existe");
        click(botaoOkAlerta);
        click(botaoCancelar);
    }

    public void editarCategoria() {
        preencherCampoProcurarTelaInicial("00004");
        alterarNomeCategoria("QUINTA CAT");

        String mensagem = getText(mensagemCantoSuperiorDireito);
        String mensagemEsperada = "Salvo com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem de salvo com sucesso, após alterar o nome da categoria");

        String nome = getText(nomeCategoriaLinha1);
        Assert.assertEquals(nome, "QUINTA CAT", "Verificação do nome da categoria após a edição");

        alterarNomeCategoria("QUATERNÁRIA");
        preencherCampoProcurarTelaInicial("00004");
        nome = getText(nomeCategoriaLinha1);
        Assert.assertEquals(nome, "QUATERNÁRIA", "Verificação do nome da categoria após alterar o nome da mesma para o valor que estava no início do teste");
    }

    public void excluirCategoria() {
        click(checkBoxCategoriaLinha1);
        click(botaoExcluir);
        tempo();

        //mensagem de alerta da exclusão
        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Deletar o(s) registro(s)?";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem de alerta da exclusão da categoria");
        click(botaoSim);

        //mensagem de excluído com sucesso
        mensagem = getText(mensagemCantoSuperiorDireito);
        mensagemEsperada = "Excluído com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem da categoria excluída com sucesso");
    }

    public void camposObrigatorios() {
        tempo();
        tempo();
        click(botaoAdicionar);
        tempo();
        click(botaoSalvar);

        //mensagem de campos obrigatório encontada nos campos
        String codigo = getText(mensagemObrigatorioCodigo);
        String nome = getText(mensagemObrigatorioNome);

        String mensagemEsperada = "Campo obrigatório";
        Assert.assertEquals(codigo, mensagemEsperada, "Verificação da mensagem de campo obrigatório, no campo de código");
        Assert.assertEquals(nome, mensagemEsperada, "Verificação da mensagem de campo obrigatório, no campo do nome");
        click(botaoCancelar);
    }

    public void tentarExcluirCategoriaComRegistrosFilhos() {
        preencherCampoProcurarTelaInicial("00001");
        click(checkBoxCategoriaLinha1);
        click(botaoExcluir);

        //mensagem de alerta da exclusão
        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Deletar o(s) registro(s)?";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem de alerta da exclusão da categoria");
        click(botaoSim);

        //mensagem de alerta informando que a exclusão não é permitida, pois existem registros filhos
        mensagem = getText(mensagemAlerta);
        mensagemEsperada = "Exclusão não permitida! O registro possui registros filhos.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem de alerta da não permissão da exclusão, devido à categoria possuir registros filhos");
        click(botaoOkAlerta);

        String codigoEncontrado = getText(codigoCategoriaLinha1);
        String nomeEncontrado = getText(nomeCategoriaLinha1);
        String codigoEsperado = "00001";
        String nomeEsperado = "PRIMÁRIA";
        Assert.assertEquals(codigoEncontrado, codigoEsperado, "Verificar se o código da categoria que possuí registros filhos não foi excluído");
        Assert.assertEquals(nomeEncontrado, nomeEsperado, "Verificar se o nome da categoria que possuí registros filhos não foi excluído");
    }

    public void tentarDeixarNomeVazio() {
        preencherCampoProcurarTelaInicial("02");
        alterarNomeCategoria("");
        String nome = getText(nomeCategoriaLinha1);
        Assert.assertEquals(nome, "", "Verificar se o nome ca categoria está vazio");
        refresh();
        preencherCampoProcurarTelaInicial("02");
        nome = getText(nomeCategoriaLinha1);
        Assert.assertEquals(nome, "SECUNDÁRIA", "Verificar se após alterar o nome da categoria deixando o campo vazio e clicar em salvar, o nome da categoria não foi alterado, permanecendo como estava antes");
    }

    private void preencherCampoProcurarTelaInicial(String texto) {
        tempo();
        tempo();
        tempo();
        tempo();
        tempo();
        if (returnClassOpened(botaoAbrirOpcoesControleTelaInicial) && !returnClassOpened(botaoPesquisarFiltro)) {
            click(botaoPesquisarFiltro);
        } else if (!returnClassOpened(botaoAbrirOpcoesControleTelaInicial)){
            click(botaoAbrirOpcoesControleTelaInicial);
            click(botaoPesquisarFiltro);
        }
        tempo();
        tempo();
        clearAndSendKeys(campoProcurar, texto);
        tempo();
        tempo();
    }

    private boolean returnClassOpened(By locator) {
        return driver.findElement(locator).getAttribute("className").contains("opened");
    }

    private void alterarNomeCategoria(String texto) {
        Actions actions = new Actions(driver);
        actions.click(driver.findElement(nomeCategoriaLinha1)).build().perform();
        tempo();
        actions.sendKeys(Keys.BACK_SPACE).build().perform();
        tempo();
        actions.sendKeys(texto).build().perform();
        tempo();
        actions.sendKeys(Keys.ENTER).build().perform();
        tempo();
        click(botaoSalvar);
    }

}
