package com.javaseleniumtemplate.pages;

import com.javaseleniumtemplate.bases.PageBase;
import org.openqa.selenium.By;
import org.testng.Assert;

public class MedidasProcionamentoPage extends PageBase {

    By botaoMenu = By.xpath("//div[@class='bars']");
    By campoPesquisar = By.xpath("//section[@class='zh-container-search-menu']//input[@type='text']");
    By opcaoTelaMedidasPorcionamento = By.xpath("//span[text() = 'Medidas de Porcionamento']");

    By botaoAdicionar = By.xpath("//div[@class=\"zh-footer-center\"]//li[@class=\"zh-ripple-effect\"]");
    By botaoSalvar = By.xpath("//div[@class=\"zh-footer-right\"]");
    By botaoEditar = By.xpath("//li[@data-zh-id=\"17672266051026083065453\"]");
    By botaoExcluir = By.xpath("//li[@data-zh-id=\"1767226605973001703455\"]");
    By botaoSim = By.xpath("//section[@class=\"zh-container-alert\"]//footer[@class=\"zh-footer-alert\"]/button[1]");
    By botaoNao = By.xpath("//section[@class=\"zh-container-alert\"]//footer[@class=\"zh-footer-alert\"]/button[2]");
    By botaoVoltar = By.xpath("//div[@class=\"zh-footer-left\"]");
    By botaoOk = By.xpath("//section[@class=\"zh-container-alert\"]//button[@type=\"submit\"]");
    By botaoCancelar = By.xpath("//div[@class=\"zh-footer-left\"]");
    By botaoAbrirOpcoesControleTelaInicial = By.xpath("//div[@data-ng-if=\"widget.isFloatingControlVisible()\"]//div[@data-zh-floating-control]");
    By botaoPesquisarFiltro = By.xpath("//div[@class=\"control-menu\"]//li[@data-zh-id=\"zh-floating-control-custom-action-1\"]");
    By campoProcurar = By.xpath("//li[@class=\"float-action expandable fix-position-bottom opened\"]//input");

    By mensagemAlerta = By.xpath("//section[@class=\"zh-container-alert\"]//div[@class=\"zh-alert-scroll\"]/p");
    By mensagemCantoSuperiorDireito = By.xpath("//*[@class=\"zh-notification-messages\"]//span[@class=\"notification-message-text\"]");
    By mensagemObrigatorioCodigo = By.xpath("//div[@data-zh-id=\"1767226605933326686443\"]//div[@class=\"zh-validation\"]/span");
    By mensagemObrigatorioNome = By.xpath("//div[@data-zh-id=\"17672266051561897484444\"]//div[@class=\"zh-validation\"]/span");
    By mensagemNenhumRegistroEncontrado = By.xpath("//div[@id=\"grid-17672266051372584218442\"]//div[@class=\"body-content\"]//p[@class=\"text-center zh-text-search-not-found ng-binding\"]");

    By campoCodigo = By.id("CDMEDICASE");
    By campoNome = By.id("NMMEDICASE");

    By codigoMedidaPorcionamentoLinha1 = By.xpath("//div[@id=\"grid-17672266051372584218442\"]//div[@class=\"body-content\"]/div[1]//div[@data-zh-field-name=\"CDMEDICASE\"]/span");
    By nomeMedidaPorcionamentoLinha1 = By.xpath("//div[@id=\"grid-17672266051372584218442\"]//div[@class=\"body-content\"]/div[1]//div[@data-zh-field-name=\"NMMEDICASE\"]/span");

    public void acessarTela() {
        tempoParaAguardarTelaInicial();
        waitForElement(botaoMenu);
        click(botaoMenu);
        sendKeys(campoPesquisar, "Medidas de Porcionamento");
        click(opcaoTelaMedidasPorcionamento);
    }

    public void adicionarMedida() {
        tempo();
        click(botaoAdicionar);
        sendKeys(campoCodigo, "10");
        sendKeys(campoNome, "COPO AMERICANO");
        click(botaoSalvar);

        preencherCampoProcurarTelaInicial("00010");
        String codigo = getText(codigoMedidaPorcionamentoLinha1);
        String nome = getText(nomeMedidaPorcionamentoLinha1);
        Assert.assertEquals(codigo, "00010", "Verificar código da medida de porcionamento adicionada");
        Assert.assertEquals(nome, "COPO AMERICANO", "Verificar nome da medida de porcionamento adicionada");
    }

    public void excluirMedida() {
        preencherCampoProcurarTelaInicial("00010");
        click(codigoMedidaPorcionamentoLinha1);
        click(botaoExcluir);

        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Deletar o(s) registro(s)?";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensagem exibida na tela após clicar para excluir a medidade de porcionamento");
        click(botaoSim);

        mensagem = getText(mensagemCantoSuperiorDireito);
        Assert.assertEquals(mensagem, "Excluído com sucesso.", "Verificar mensagem de excluído com sucesso exibida no canto superior direito da tela");

        mensagem = getText(mensagemNenhumRegistroEncontrado);
        mensagemEsperada = "Nenhum registro foi encontrado";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensagem exibida no grid com a lista de medidas, informando que nenhum registro foi encontrado");
    }

    public void editarNomeMedida() {
        preencherCampoProcurarTelaInicial("00004");
        click(codigoMedidaPorcionamentoLinha1);
        click(botaoEditar);

        boolean campoCodigoHabilitado = returnIfElementIsEnabled(campoCodigo);
        Assert.assertEquals(campoCodigoHabilitado, false, "Verificar se durante a edição do nome de uma medida, o campo código está habilitado");
        clearAndSendKeys(campoNome, "COPO DE REQUEIJÃO");
        click(botaoSalvar);

        refresh();
        preencherCampoProcurarTelaInicial("00004");
        tempo();
        tempo();
        String nome = getText(nomeMedidaPorcionamentoLinha1);
        Assert.assertEquals(nome, "COPO DE REQUEIJÃO", "Verificar se o nome da medidade de porcionamento foi alterado corretamente");

        click(codigoMedidaPorcionamentoLinha1);
        click(botaoEditar);
        clearAndSendKeys(campoNome, "COLHER DE CHÁ");
        click(botaoSalvar);
        click(botaoVoltar);
        nome = getText(nomeMedidaPorcionamentoLinha1);
        Assert.assertEquals(nome, "COLHER DE CHÁ", "Verificar se o nome foi alterado para o nome que estava no início do teste");
    }

    public void tentarAdicionarCodigoExistente() {
        click(botaoAdicionar);
        sendKeys(campoCodigo, "3");
        sendKeys(campoNome, "COLHER");
        click(botaoSalvar);

        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Impossível adicionar. Registro já existe.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificar mensagem exibida na tela, informando que registro já existe");
        click(botaoOk);
        click(botaoVoltar);
    }

    public void camposObrigatorios() {
        tempo();
        click(botaoAdicionar);
        tempo();
        click(botaoSalvar);

        String codigo = getText(mensagemObrigatorioCodigo);
        String nome = getText(mensagemObrigatorioNome);
        String mensagemEsperada = "Campo obrigatório";
        Assert.assertEquals(codigo, mensagemEsperada, "Verificar mensagem de campo obrigatório no campo código ao tentar adicionar uma medida");
        Assert.assertEquals(nome, mensagemEsperada, "Verificar mensagem de campo obrigatório no campo nome ao tentar adicionar uma medida");
        click(botaoCancelar);

        tempo();
        click(codigoMedidaPorcionamentoLinha1);
        click(botaoEditar);
        clear(campoNome);
        click(botaoSalvar);
        nome = getText(mensagemObrigatorioNome);
        Assert.assertEquals(nome, mensagemEsperada, "Verificar mensagem de campo obrigatório no campo nome, no processo de editar uma medida");
        click(botaoCancelar);
    }

    public void cancelarExclusao() {
        preencherCampoProcurarTelaInicial("00003");
        click(codigoMedidaPorcionamentoLinha1);
        click(botaoExcluir);

        String mensagem = getText(mensagemAlerta);
        Assert.assertEquals(mensagem, "Deletar o(s) registro(s)?", "Verificação da mensagem de alerta exibida após excluir a medida");
        click(botaoNao);
        click(botaoVoltar);

        String codigo = getText(codigoMedidaPorcionamentoLinha1);
        String nome = getText(nomeMedidaPorcionamentoLinha1);
        Assert.assertEquals(codigo, "00003", "Verificação do código da medida de porcionamento após cancelar a exclusão. O código está sendo conferido na tela inicial");
        Assert.assertEquals(nome, "COLHER DE SOPA", "Verificação do nome da medida de porcionamento após cancelar a exclusão. O nome está sendo conferido na tela inicial");
    }

    private void preencherCampoProcurarTelaInicial(String texto) {
        tempo();
        tempo();
        tempo();
        tempo();
        tempo();
        if (returnClassOpened(botaoAbrirOpcoesControleTelaInicial) && !returnClassOpened(botaoPesquisarFiltro)) {
            click(botaoPesquisarFiltro);
        } else if (!returnClassOpened(botaoAbrirOpcoesControleTelaInicial)){
            click(botaoAbrirOpcoesControleTelaInicial);
            click(botaoPesquisarFiltro);
        }
        clearAndSendKeys(campoProcurar, texto);
        tempo();
    }

    private boolean returnClassOpened(By locator) {
        return driver.findElement(locator).getAttribute("className").contains("opened");
    }

}
